@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 """ROCm_SMI_LIB CLI Tool Python Bindings"""
+# NOTE: You MUST call rsmiBindings.initRsmiBindings() when using this library!
 # TODO: Get most (or all) of these from rocm_smi.h to avoid mismatches and redundancy
 
 from __future__ import print_function
@@ -11,40 +12,54 @@ import os
 
 # Use ROCm installation path if running from standard installation
 # With File Reorg rsmiBindings.py will be installed in  /opt/rocm/libexec/rocm_smi.
-# relative path changed accordingly
-path_librocm = os.path.dirname(os.path.realpath(__file__)) + '/../../lib/librocm_smi64.so'
-if not os.path.isfile(path_librocm):
-    print('Unable to find %s . Trying /opt/rocm*' % path_librocm)
-    for root, dirs, files in os.walk('/opt', followlinks=True):
-        if 'librocm_smi64.so' in files:
-            path_librocm = os.path.join(os.path.realpath(root), 'librocm_smi64.so')
-    if os.path.isfile(path_librocm):
-        print('Using lib from %s' % path_librocm)
+# relative path changed accordingly.
+# if ROCM_SMI_LIB_PATH is set, we can load 'librocm_smi64.so' from that location
+#
+# Library load is wrapped in a function so prints can be hidden for PRINT_JSON mode.
+path_librocm = str()
+def initRsmiBindings(silent=False):
+    def print_silent(*args):
+        if not silent:
+            print(args)
+
+    rocm_smi_lib_path = os.getenv('ROCM_SMI_LIB_PATH')
+    if (rocm_smi_lib_path != None):
+        path_librocm = rocm_smi_lib_path
     else:
-        print('Unable to find librocm_smi64.so')
+        path_librocm = os.path.dirname(os.path.realpath(__file__)) + '/../../lib/librocm_smi64.so.1'
 
-# ----------> TODO: Support static libs as well as SO
-
-try:
-    cdll.LoadLibrary(path_librocm)
-    rocmsmi = CDLL(path_librocm)
-except OSError:
-    print('Unable to load the rocm_smi library.\n'\
-          'Set LD_LIBRARY_PATH to the folder containing librocm_smi64.\n'\
-          '{0}Please refer to https://github.com/'\
-          'RadeonOpenCompute/rocm_smi_lib for the installation guide.{1}'\
-          .format('\33[33m', '\033[0m'))
-    exit()
+    if not os.path.isfile(path_librocm):
+        print_silent('Unable to find %s . Trying /opt/rocm*' % path_librocm)
+        for root, dirs, files in os.walk('/opt', followlinks=True):
+            if 'librocm_smi64.so.1' in files:
+                path_librocm = os.path.join(os.path.realpath(root), 'librocm_smi64.so.1')
+        if os.path.isfile(path_librocm):
+            print_silent('Using lib from %s' % path_librocm)
+        else:
+            print('Unable to find librocm_smi64.so.1')
 
+    # ----------> TODO: Support static libs as well as SO
+    try:
+        cdll.LoadLibrary(path_librocm)
+        return CDLL(path_librocm)
+    except OSError:
+        print('Unable to load the rocm_smi library.\n'\
+              'Set LD_LIBRARY_PATH to the folder containing librocm_smi64.so.1\n'\
+              '{0}Please refer to https://github.com/'\
+              'RadeonOpenCompute/rocm_smi_lib for the installation guide.{1}'\
+              .format('\33[33m', '\033[0m'))
+        exit()
 
 # Device ID
 dv_id = c_uint64()
 # GPU ID
 gpu_id = c_uint32(0)
 
+SMI_HASH = 'unknown'
+
 
 # Policy enums
-RSMI_MAX_NUM_FREQUENCIES = 32
+RSMI_MAX_NUM_FREQUENCIES = 33
 RSMI_MAX_FAN_SPEED = 255
 RSMI_NUM_VOLTAGE_CURVE_POINTS = 3
 
@@ -404,25 +419,27 @@ class rsmi_fw_block_t(c_int):
     RSMI_FW_BLOCK_ME = 4
     RSMI_FW_BLOCK_MEC = 5
     RSMI_FW_BLOCK_MEC2 = 6
-    RSMI_FW_BLOCK_PFP = 7
-    RSMI_FW_BLOCK_RLC = 8
-    RSMI_FW_BLOCK_RLC_SRLC = 9
-    RSMI_FW_BLOCK_RLC_SRLG = 10
-    RSMI_FW_BLOCK_RLC_SRLS = 11
-    RSMI_FW_BLOCK_SDMA = 12
-    RSMI_FW_BLOCK_SDMA2 = 13
-    RSMI_FW_BLOCK_SMC = 14
-    RSMI_FW_BLOCK_SOS = 15
-    RSMI_FW_BLOCK_TA_RAS = 16
-    RSMI_FW_BLOCK_TA_XGMI = 17
-    RSMI_FW_BLOCK_UVD = 18
-    RSMI_FW_BLOCK_VCE = 19
-    RSMI_FW_BLOCK_VCN = 20
+    RSMI_FW_BLOCK_MES = 7
+    RSMI_FW_BLOCK_MES_KIQ = 8
+    RSMI_FW_BLOCK_PFP = 9
+    RSMI_FW_BLOCK_RLC = 10
+    RSMI_FW_BLOCK_RLC_SRLC = 11
+    RSMI_FW_BLOCK_RLC_SRLG = 12
+    RSMI_FW_BLOCK_RLC_SRLS = 13
+    RSMI_FW_BLOCK_SDMA = 14
+    RSMI_FW_BLOCK_SDMA2 = 15
+    RSMI_FW_BLOCK_SMC = 16
+    RSMI_FW_BLOCK_SOS = 17
+    RSMI_FW_BLOCK_TA_RAS = 18
+    RSMI_FW_BLOCK_TA_XGMI = 19
+    RSMI_FW_BLOCK_UVD = 20
+    RSMI_FW_BLOCK_VCE = 21
+    RSMI_FW_BLOCK_VCN = 22
     RSMI_FW_BLOCK_LAST = RSMI_FW_BLOCK_VCN
 
 
 # The following list correlated to the rsmi_fw_block_t
-fw_block_names_l = ['ASD', 'CE', 'DMCU', 'MC', 'ME', 'MEC', 'MEC2', 'PFP',\
+fw_block_names_l = ['ASD', 'CE', 'DMCU', 'MC', 'ME', 'MEC', 'MEC2', 'MES', 'MES KIQ', 'PFP',\
                     'RLC', 'RLC SRLC', 'RLC SRLG', 'RLC SRLS', 'SDMA', 'SDMA2',\
                     'SMC', 'SOS', 'TA RAS', 'TA XGMI', 'UVD', 'VCE', 'VCN']
 
@@ -477,7 +494,8 @@ rsmi_power_profile_status = rsmi_power_profile_status_t
 
 
 class rsmi_frequencies_t(Structure):
-    _fields_ = [('num_supported', c_int32),
+    _fields_ = [('has_deep_sleep', c_bool),
+                ('num_supported', c_int32),
                 ('current', c_uint32),
                 ('frequency', c_uint64 * RSMI_MAX_NUM_FREQUENCIES)]
 
@@ -618,24 +636,29 @@ rsmi_compute_partition_type = rsmi_compute_partition_type_t
 # will return string 'CPX'
 compute_partition_type_l = ['CPX', 'SPX', 'DPX', 'TPX', 'QPX']
 
-class rsmi_nps_mode_type_t(c_int):
+class rsmi_memory_partition_type_t(c_int):
     RSMI_MEMORY_PARTITION_UNKNOWN = 0
     RSMI_MEMORY_PARTITION_NPS1 = 1
     RSMI_MEMORY_PARTITION_NPS2 = 2
     RSMI_MEMORY_PARTITION_NPS4 = 3
     RSMI_MEMORY_PARTITION_NPS8 = 4
 
-rsmi_nps_mode_type_dict = {
+rsmi_memory_partition_type_dict = {
     'NPS1': 1,
     'NPS2': 2,
     'NPS4': 3,
     'NPS8': 4
 }
 
-rsmi_nps_mode_type = rsmi_nps_mode_type_t
+rsmi_memory_partition_type = rsmi_memory_partition_type_t
 
-# nps_mode_type_l includes string names for the rsmi_compute_partition_type_t
+# memory_partition_type_l includes string names for the rsmi_compute_partition_type_t
 # Usage example to get corresponding names:
-# nps_mode_type_l[rsmi_nps_mode_type_t.RSMI_MEMORY_PARTITION_NPS2]
+# memory_partition_type_l[rsmi_memory_partition_type_t.RSMI_MEMORY_PARTITION_NPS2]
 # will return string 'NPS2'
-nps_mode_type_l = ['NPS1', 'NPS2', 'NPS4', 'NPS8']
\ No newline at end of file
+memory_partition_type_l = ['NPS1', 'NPS2', 'NPS4', 'NPS8']
+
+class rsmi_power_label(str, Enum):
+    AVG_POWER = '(Avg)'
+    CURRENT_SOCKET_POWER = '(Socket)'
+
